var backend = "api.battlefront.dice.se"; // DO_NOT_MODIFY
var appId = "bb5886e89656560f50324dc34b1abe94"; // DO_NOT_MODIFY
var staticHost = "static.battlefront.dice.se"; // DO_NOT_MODIFY

var gcmSenderId = "758252314540";

// Override default back button functionality to prevent potential crashes when user clicks back during app load
document.addEventListener("backbutton", function () {}, false);

if (localStorage) {
    var backendOverride = localStorage.getItem("backendOverride");

    if (backendOverride) {
        backend = backendOverride;
    }
}

window.gaEventQueue = [];

window.battlefront = window.battlefront || {};

window.battlefront.reloadApp = function () {
    // Special handling to get app reload work, index.html/x/y/z doesn't load in native app
    history.replaceState({}, "", document.location.pathname.replace("index.html", "index.html#"));
    document.location.reload();
};

// Setup app reloader
window.onload = function () {
    var element = document.getElementById("reloader");
    element.addEventListener("click", window.battlefront.reloadApp);
    element.addEventListener("touchend", window.battlefront.reloadApp);
};

window.battlefront.initAppErrorHandler = function (errorCode) {
    errorCode = errorCode || "";
    navigator.splashscreen.hide();
    // Default to "en_US" if no phone locale has been set
    var locale = window.battlefront.phoneLocale || "en_US";
    var lang = locale.substr(0, 2);
    document.location.href = "error.html?lang=" + lang + "&error=" + errorCode;
};

document.addEventListener("deviceready", function () {
    if (window.chrome && window.chrome.system && window.chrome.system.memory){
        var logUpdateInterval = localStorage.getItem("logMemoryInterval") || 2000,
            logUpdateItv = null,
            appMemoryItv = null,
            appMemory = 0;

        var formatBytes = function(bytes){
            return (Math.round((bytes / 1024 / 1024) * 100) / 100) + "Mb";
        };
        window.chrome.system.memory.setEnabled = function(state){
            if (state){
                var div = document.createElement("div");
                document.body.appendChild( div );
                div.setAttribute("id", "log-memory-container");
                div.style.position = "fixed";
                div.style.top = "5px";
                div.style.left = "5px";
                div.style.zIndex = "99999";
                div.style.padding = "10px";
                div.style.backgroundColor = "#fff";
                div.style.fontSize = "12px";
                div.style.lineHeight = "18px";
                div.innerText = "";

                div.addEventListener('click', function() {
                    if (div.style.left == "5px") {
                        div.style.right = "5px";
                        div.style.left = "inherit";
                    } else {
                        div.style.left = "5px";
                        div.style.right = "inherit";
                    }
                }, false);

                var updateMemoryInfo = function() {
                    window.chrome.system.memory.getInfo(function(obj){
                        var per = Math.round((obj.availableCapacity / obj.capacity) * 10000) / 100;
                        var str = per + " % &nbsp; <span style='color: #444;'>("+ formatBytes(obj.availableCapacity) + " of " + formatBytes(obj.capacity) +")</span>";
                        if (appMemory) {
                            str += "<br>App usage: " + appMemory;
                        }
                        div.innerHTML = str;
                        logUpdateItv = setTimeout(updateMemoryInfo, logUpdateInterval);
                    });
                };

                var updateAppMemoryInfo = function() {
                    window.cordovautils.getAppMemory(function (bytes) {
                        appMemory = formatBytes(parseInt(bytes));
                        appMemoryItv = setTimeout(updateAppMemoryInfo, 2000);
                    });
                };

                console.log("***** Starting memory log *****");
                console.log("Update interval: "+ logUpdateInterval +" ms");
                logUpdateItv = setTimeout(updateMemoryInfo, logUpdateInterval);

                if (window.cordovautils){
                    appMemoryItv = setTimeout(updateAppMemoryInfo, 2000);
                }

            } else {
                if (logUpdateItv){
                    clearInterval(logUpdateItv);
                }
                if (appMemoryItv){
                    clearInterval(appMemoryItv);
                }
                var element = document.getElementById("log-memory-container");
                if (element){
                    element.parentNode.removeChild(element);
                }
            }
        }
    }

    if (window.nimble){
        function storeMobilePushId (response) {
            if (response.info && response.info.deviceId) {
                localStorage.setItem("mobilePushId", response.info.deviceId);
            }
        }
        console.log("Setup nimble");
        window.nimble.setup(function(response){
            if (!response) {
                window.gaEventQueue.push([
                    "Nimble",
                    "Setup Failed"
                ]);
                console.log("Nimble: setup failed");
                return;
            }
            console.log("Nimble response", response);

            switch (response.callbackType) {
                case "setupSuccess":
                    console.log("Nimble: Setup successfully");
                    storeMobilePushId(response);
                    break;

                case "setupSuccessWithPushNotification":
                    storeMobilePushId(response);
                    // The app was started through a push notification. Store the push notification object since
                    // the app isn't ready to handle it yet. It will be dealed with when the MobilRoot is mounted.
                    window.nimble.storedPushNotification = response.data;
                    break;

                case "synergyReady":
                    storeMobilePushId(response);
                    break;

                case "pushNotificationClicked":
                    localStorage.setItem("launchSource", "pn");
                    window.dispatchEvent( new CustomEvent("pushNotificationClicked", { "detail": response }) );
                    break;

                case "pushRegistrationSuccess":
                    window.dispatchEvent( new CustomEvent("pushRegistrationSuccess", { "detail": response.data }) );
                    break;

                case "pushRegistrationError":
                    window.dispatchEvent( new CustomEvent("pushRegistrationError", { "detail": response.data }) );
                    break;
            }

        }, function(){
            window.gaEventQueue.push([
                "Nimble",
                "Error During Setup"
            ]);
            console.log("Nimble: error during setup");
        });
    } else {
        window.gaEventQueue.push([
            "Nimble",
            "Plugin Missing"
        ]);
        console.log("Could not find Nimble plugin");
    }


    // Minimize the app when pressing back button from root page on android
    if (cordova.platformId == "android" && navigator.Backbutton) {
        document.addEventListener("backbutton", function(e){
            if (window.location.href.substr(-10) == 'index.html') {
                e.preventDefault();
                e.stopPropagation();
                navigator.Backbutton.goHome();
            }
        }, true);
    }


    if (appId && window.hockeyapp) {
        var autoSendCrashReports = (localStorage.getItem("crashReports") !== "0");
        window.hockeyapp.start(
            function success () {
                // Should we await app init until HockeyApp SDK has successfully started?
            },
            function error (msg) {
                if (msg != "hockeyapp cordova plugin: plugin is already started!") {
                    console.error("HockeyApp SDK didn't start correctly", msg);
                }
            },
            appId,
            autoSendCrashReports
        );
    } else {
        console.warn("Could not start hockeyapp SDK.", window.hockeyapp ? "appId" : "hockeyapp", "not defined");
    }

    // Setup Lookback plugin
    if (window.Lookback) {
        window.Lookback.setupWithAppToken("JEHvGgNeWC66A4wMT");
        window.Lookback.shakeToRecord(true);

        if (device) {
            window.Lookback.userIdentifier(device.uuid);
        }
    }


    // If on iOS, check if there is enough available disk space
    if (window.cordovautils){
        window.cordovautils.getFreeSpace(function (freeBytes) {
            var DISK_SPACE_LIMIT = 50 * 1024 * 1024;
            if (freeBytes < DISK_SPACE_LIMIT) {
                window.battlefront.initAppErrorHandler("nodiskspace");
                return;
            }
        }, function () {
            console.log("Error checking for available disk space");
        });
    }

    // Check if we're connected to the interweb
    if (navigator.connection && window.Connection){
        var networkState = navigator.connection.type;
        if (networkState == window.Connection.NONE){
            window.battlefront.initAppErrorHandler("noconnection");
            return;
        }
    }

    // Hide the splashscreen asap, so we can show a HTML loading screen instead
    if (navigator.splashscreen) {
        navigator.splashscreen.hide();
    }

    // Check if background music is playing
    if (window.backgroundmusic) {
        window.backgroundmusic.setupChangeListener(function (status) {
            if (status == "backgroundmusicstarted") {
                window.backgroundmusic._isPlaying = true;
                document.dispatchEvent(new CustomEvent("backgroundmusicchanged", { detail: { "isPlaying": true } }));
            } else if (status == "backgroundmusicstopped") {
                window.backgroundmusic._isPlaying = false;
                document.dispatchEvent(new CustomEvent("backgroundmusicchanged", { detail: { "isPlaying": false } }));
            }
        }, function () {
            console.log("Error setting up backgroundmusic events")
        });

        function checkIfBackgroundMusicIsPlaying () {
            window.backgroundmusic.isPlaying(function(bol){
                if (window.backgroundmusic._isPlaying !== bol) {
                    window.backgroundmusic._isPlaying = bol;
                    var event = new CustomEvent("backgroundmusicchanged", { detail: { "isPlaying": bol } });
                    document.dispatchEvent(event);
                }
            }, function(){
                console.log("Error checking for background music")
            });
        }

        document.addEventListener("resume", checkIfBackgroundMusicIsPlaying, false);
    }

    var loadScript = function (url, onload) {
        var script = document.createElement("script");
        script.onload = function () { onload(); };
        script.onerror = function () {
            console.error("Script " + url + " failed to load");
            window.battlefront.initAppErrorHandler("script_load_error");
        };
        script.src = url;
        document.head.appendChild(script);
    };

    var jsDir = "http://" + backend + "/starwars/battlefront/resources/js/";

    // Load init app script from backend
    loadScript(jsDir + "init.js", function () {
        // Fetch device's system language and load corresponding bundle
        navigator.globalization.getPreferredLanguage(
            function success (locale) {
                // Map system language to locale (map comes from backend)
                var lowercaseLocale = locale.value.toLowerCase();
                var language = locale.value.split("-")[0];
                var gameLanguageMapping = window.battlefront.gameLanguageMapping;
                var languageMapping = window.battlefront.languageMapping;

                // First check if it can be mapped to the game language mapping
                if (gameLanguageMapping && gameLanguageMapping[lowercaseLocale]) {
                    window.battlefront.phoneLocale = gameLanguageMapping[lowercaseLocale];
                // If not, try to only get the language and map that
                } else if (languageMapping && languageMapping[language]) {
                    window.battlefront.phoneLocale = languageMapping[language];
                } else {
                    // Default locale to en_US
                    window.battlefront.phoneLocale = "en_US";
                }

                // Handle possible deep link
                if (document.location.hash) {
                    history.replaceState({}, "", document.location.pathname + document.location.hash.substr(1));
                }

                window.battlefront.initApp(backend);
            },
            function error () {
                window.battlefront.initApp(backend);
            }
        );
    });
}, false);
